#!/usr/bin/perl

# CGI File Browser & Image Gallery
# Copyright 2000 Eye of Dog, Inc.

# Author:     Alex Shinn <foof@synthcode.com>
# Created:    <Tue Nov 23 12:00:00 EST 1999>
# Time-stamp: <2003-07-26 00:23:52 foof>

use strict;
use CGI;

#################
# Customizations
#################

my $width = 120;   # default thumbnail width
my $height = 100;  # default thumbnail height
my $max_rows = 3;  # max number of rows of files/images
my $max_cols = 3;  # max number of cols of files/images

my $show_images = 1;
my $show_files = 0;
my $show_dirs = 1;

my $doc_title = "Image Gallery";
my $doc_keywords = "image, gallery";
my $doc_stylesheet = "/styles/default.css";

my $base_dir = "/gallery";
my $icon_dir = "/images";
my $serverroot = "/var/www";

my %icons = (
     # Special Icons
     dir  => "dir.gif",
     file => "file.gif",
     up   => "up.gif",
     prev => "prev.gif",
     next => "next.gif",
     # Extension-based Icons
     mp3  => "mp3.gif",
     pdf  => "pdf.gif"
);


##################
# Initializations
##################

my $query = new CGI;
my $myself = $query->self_url;
$myself =~ s/\?.*$//;
my $scroll  = $query->url(-relative=>1);
my $max_files = $max_rows * $max_cols;
my $max_pages = 0;


# Interpret the CGI parameters - grabs three parameters, page and
# index (forced to an int), and dir (with ..'s and other special chars
# removed).

my $index = $query->param('index');
$index = int($index) if (defined $index);
my $page = $query->param('page');
$page = int($page);

my $http_dir = $query->param('dir');
$http_dir =~ s/\.\.+//g;
$http_dir =~ s/[\'\`~,!\@\#\$\%\^\&\*\(\)\"\;]//g;
my $dir = $http_dir;
$dir =~ s/\%2f/\//gi;
my $mydir = $myself;
$mydir =~ s!/[^/]*$!!;
$mydir .= ($dir ? "$dir/" : "");
my $pwd = $base_dir . ($dir ? "/$dir" : "");
my $parent_dir = $dir;
$parent_dir =~ s/\/[^\/]*$//;
$parent_dir =~ s/\//\%2f/g;


# Get the directory listing

my @listings = glob(($pwd =~ /^\//) ? "$serverroot/$pwd/*" : "$pwd/*");
my @images = ();
my @files = ();
my @subdirs = ();
my %thumbnails = ();


# Separate the images, thumbnails, directories, and other files

foreach my $f (@listings) {
  if ($f =~ /.*_t\.(jpe?g|gif|png|tiff|bmp)/i) {
    my $base = $f;
    $base =~ s/_t\.[^.]*$//;
    $thumbnails{$base} = $f;
  }
  elsif ($f =~ /.*\.(jpe?g|gif|png|tiff|bmp)/i) {
    push @images, $f;
  }
  elsif (-d $f) {
    push @subdirs, $f;
  }
  else {
    push @files, $f;
  }
}


#########
# Output
#########

# Print the start of our HTML document

print  $query->header,
  $query->start_html(-title=>$doc_title,
                     -author=>'foof@synthcode.com',
                     -meta=>{'keywords'=>$doc_keywords},
                     -style=>{'src'=>$doc_stylesheet});

# Print the files

if (defined $index) {
  # Print the image
  print_image();
  # Print the navigation buttons
  print_image_nav();
}
else {
  # Browsing a directory
  print_files() if ($show_images or $show_files);
  # Print the subdirectories
  print_dirs() if $show_dirs;
  # Print the navigation buttons
  print_nav() if ($show_images or $show_files);
}

# End this document

print $query->end_html;



##############
# Subroutines
##############


# Print the files

sub print_files {
  my $n = $page * $max_files;
  my ($i, $j);
  my @list = ($show_images and $show_files) ? (@images, @files) :
    ($show_images ? @images : @files);
  $max_pages = int($#list / $max_files);
  print "<table>\n";
  if ($show_images) {
    for ($i=0; $i < $max_rows && $n <= $#images; $i++) {
      print "<tr>\n";
      for ($j=0; $j < $max_cols && $n <= $#images; $j++, $n++) {
        my $file = $images[$n];
        $file =~ s/^.*\///;
        my $basename = $file;
        $basename =~ s/\.[^.]*$//;
        my $name = $basename;
        $name =~ s/_/ /g;
        $name = "\u$name";
        if ($thumbnails{$basename}) {
          print "<td width=\"$width\" height=\"$height\" align=\"center\">",
          "<a href=\"$myself\?index=$n", ($dir ? "&dir=$dir\">" : "\">"),
          "<img src=\"", ($pwd ? "$pwd/" : ""),
          "$thumbnails{$basename}\" alt=\"$name\">",
          "</a></td>\n";
        }
        else {
          print "<td width=\"$width\" height=\"$height\" align=\"center\">",
          "<a href=\"$myself\?index=$n", ($dir ? "&dir=$dir\">" : "\">"),
          "<img src=\"$pwd/$file\" alt=\"$name\" width=\"$width\" ",
          "height=\"$height\"></a></td>\n";
        }
      }
      if ($j == $max_cols) {
        print "</tr>\n";
        if ($show_files) {
          print "<tr>\n";
        }
      }
    }
  }
  if ($show_files) {
    for ( ; $i < $max_rows && $n <= $#files; $i++) {
      for ( ; $j < $max_cols && $n <= $#files; $j++, $n++) {
        my $file = $files[$n-$#images];
        $file =~ s/^.*\///;
        my $basename = my $ext = $file;
        $basename =~ s/\.[^.]*$//;
        $ext =~ s/^[^.]*\.//;
        my $name = $basename;
        $name =~ s/_/ /g;
        $name = "\u$name";
        if ($thumbnails{$basename}) {
          print "<td width=\"$width\" height=\"$height\" align=\"center\">",
          "<a href=\"$mydir/$file\">",
          "<img src=\"$dir/$thumbnails{$basename}\" alt=\"$name\">",
          "</a></td>\n";
        }
        else {
          print "<td width=\"$width\" height=\"$height\" align=\"center\">",
          "<a href=\"$dir/$file\">",
          "<img src=\"" . ($icons{$ext} or $icons{$file}),
          "\" alt=\"$name\" width=\"$width\" ",
          "height=\"$height\"></a></td>\n";
        }
      }
      print "</tr>\n";
      print "<tr>\n" if ($i < $max_rows and $n <= $#files);
    }
    print "</tr>\n";
  }
  print "</table>\n";
}


# Print the directories

sub print_dirs {
  my $url = "";
  my $icon = "";
  if ($dir) {
    $url = $myself . ($parent_dir ? "\?dir=$parent_dir" : "");
    $icon = $thumbnails{'up'} ? $dir/$thumbnails{'up'} :
      "$icon_dir/$icons{'up'}";
    print $query->p("<a href=\"$url\">" .
                    "<img src=\"$icon\" alt=\"Up\"></a>");
  }
  if (@subdirs) {
    print "<p><table>\n  <tr>\n";
    # Print each sub-dir
    foreach my $d (@subdirs) {
      $url = "$myself\?dir=" . ($http_dir ? "$http_dir\%2f$d" : $d);
      $icon = $thumbnails{$d} ? $dir/$thumbnails{$d} :
        "$icon_dir/dir.gif";
      print $query->td("<a href=\"$url\">" . 
                       "<img src=\"$icon\" alt=\"$d\"><br>$d</a>");
    }
    print "  </tr>\n</table></p>\n";
  }
}


# Print the navigation buttons

sub print_nav {
  my $i = 0;

  print "<p>\n<table>\n";

  if ($page) {
    print $query->td("<a href=\"$scroll\?page="
                     . ($page - 1)
                     . "\"><img src=\"$icon_dir/left.png\" "
                     . "alt=\"\" border=0><br>&lt;&lt;</a>");
  }

  for ($i=0; $i < $page; $i++) {
    print $query->td("<a href=\"$scroll\?page=$i\"><img src=\"$icon_dir/link.png\" "
                     . "alt=\"\" border=0><br>$i</a>");
  }

  print $query->td("<img src=\"$icon_dir/current.png\" alt=\"\" border=0><br>$i");

  for ($i=$page+1; $i <= $max_pages; $i++) {
    print $query->td("<a href=\"$scroll\?page=$i\"><img src=\"$icon_dir/link.png\" "
                     . "alt=\"\" border=0><br>$i</a>");
  }

  if ($page < $max_pages) {
    print $query->td("<a href=\"$scroll\?page="
                     . ($page + 1)
                     . "\"><img src=\"$icon_dir/right.png\" alt=\"\" border=0>"
                     . "<br>&gt;&gt;</a>");
  }

  print "</table>\n</center>\n";
}


sub print_image {
  # Print the image
  my $file = $images[$index];
  $file =~ s/^.*\///;
  my $basename = $file;
  $basename =~ s/\.[^.]*$//;
  my $name = $basename;
  $name =~ s/_/ /g;
  $name = "\u$name";
  print "<center><p>\n";
  print "<img src=\"$pwd/$file\" align=\"center\" alt=\"$name\"></p>\n";
}


sub print_image_nav {
  # Print the image navigation buttons
  print "<p>\n";
  print $query->table($query->td("<a href=\"$myself\?index="
                                 . ($index ? ($index - 1) : $#images)
                                 . "\"><img src=\"$icons{'prev'}\" "
                                 . "alt=\"Prev\"></a>"),
                      $query->td("<a href=\"$myself\?page="
                                 . int($index / $max_files)
                                 . "\"><img src=\"$icons{'up'}\" "
                                 . "alt=\"Up\"></a>"),
                      $query->td("<a href=\"$scroll\?index="
                                 . (($index + 1) % $#images)
                                 . "\"><img src=\"$icons{'next'}\" "
                                 . "alt=\"Next\"></a>"));
  print "</center></p>\n";
}
